/** OpenCP Module Player
 * copyright (c) '94-'05 Niklas Beisert <nbeisert@physik.tu-muenchen.de>
 *
 * Jack Player device
 *
 *  revision history: (please note changes here)
 */

#include "config.h"
#include <jack/jack.h>
#include <stdio.h>

static void my_jack_error_callback(const char *msg)
{
	fprintf(stderr, "JACK: %s\n", msg);
}
extern void default_jack_error_callback(const char *); /* needed in old jack versions */

static int my_jack_process_callback(jack_nframes_t nframes, void *arg)
{
	return 0; /* no error */
	/*return 1;*/ /* error */
}

static int my_jack_set_buffer_size_callback(jack_nframes_t nframes, void *arg)
{
	fprintf(stderr, "set_buffer_size: %d\n", nframes);
	return 0; /* no error */
	/*return 1;*/ /* error */
}

static int my_jack_set_sample_rate_callback(jack_nframes_t nframes, void *arg)
{
	fprintf(stderr, "set_sample_rate: %d\n", nframes);
	return 0; /* no error */
	/*return 1;*/ /* error */
}

int main(int argc, char *argv[])
{
	jack_client_t *jack;
	jack_port_t *leftport, *rightport;
	
	jack_set_error_function(my_jack_error_callback);
	if (!(jack = jack_client_new("Open Cubic Player")))
	{
		fprintf(stderr, "jack_client_new() failed\n");	
		jack_set_error_function(default_jack_error_callback);
		return -1;
	}

	if (jack_set_process_callback(jack, my_jack_process_callback, NULL))
	{
		fprintf(stderr, "jack_set_process_callback() failed\n");
	
		if (jack_client_close(jack))
			fprintf(stderr, "jack_client_close() failed\n");

		jack_set_error_function(default_jack_error_callback);
		return -1;
	}

	if (jack_set_buffer_size_callback(jack, my_jack_set_buffer_size_callback, NULL))
	{
		fprintf(stderr, "jack_set_buffer_size_callback() failed\n");
	
		if (jack_client_close(jack))
			fprintf(stderr, "jack_client_close() failed\n");


		jack_set_error_function(default_jack_error_callback);
		return -1;
	}

	if (jack_set_sample_rate_callback(jack, my_jack_set_sample_rate_callback, NULL))
	{
		fprintf(stderr, "jack_set_sample_rate_callback() failed\n");
		if (jack_client_close(jack))
			fprintf(stderr, "jack_client_close() failed\n");


		jack_set_error_function(default_jack_error_callback);
		return -1;
	}

	fprintf(stderr, "Default sample rate is %d\n", jack_get_sample_rate(jack));

	if (!(leftport = jack_port_register(jack, "pcm out, left", JACK_DEFAULT_AUDIO_TYPE, JackPortIsOutput, 0)))
	{
		fprintf(stderr, "jack_port_register() failed\n");
		if (jack_client_close(jack))
			fprintf(stderr, "jack_client_close() failed\n");

		jack_set_error_function(default_jack_error_callback);
		return -1;
	}

	if (!(rightport = jack_port_register(jack, "pcm out, right", JACK_DEFAULT_AUDIO_TYPE, JackPortIsOutput, 0)))
	{
		fprintf(stderr, "jack_port_register() failed\n");

		if (jack_port_unregister(jack, leftport))
			fprintf(stderr, "jack_port_unregister() failed\n");
	
		if (jack_client_close(jack))
			fprintf(stderr, "jack_client_close() failed\n");

		jack_set_error_function(default_jack_error_callback);
		return -1;
	}

	fprintf(stderr, "So far, so good\n");
	
	if (jack_port_unregister(jack, leftport))
		fprintf(stderr, "jack_port_unregister() failed\n");
	
	if (jack_client_close(jack))
		fprintf(stderr, "jack_client_close() failed\n");

	jack_set_error_function(default_jack_error_callback);
	return 0;
}
